/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia. 
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* maintenance code (dosa port detection)
*
* code to detect port scanning. Tally port connections of each
* source->destination pair.. if a given S-D pair have more port 
* connections than the limit, then it triggers.
*
**************************************************************************/
#include <curses.h>
#include "defs.h"
#include "lib.p"
#include "iptrk.p"

typedef struct
{
  addr_key_t addr;
  int        count;
} port_tally_t;

PRIVATE dosa_t *dosp;
PRIVATE int    tally;

/**************************************************************************
*
**************************************************************************/
PRIVATE void scan_port_tree(treap_t *t)
{
  if(t == NULL)
    return;

  scan_port_tree(t->l);
  tally++;
  scan_port_tree(t->r);
}

PRIVATE void count_ports(iptrk_t *n)
{
  struct in_addr in;
  char           str[STR_LEN + 1],
                 str1[STR_LEN + 1],
                 email_body[STR_LEN_HUGE];

  tally = 0;
  scan_port_tree(n->tcp_port_ll);
  scan_port_tree(n->udp_port_ll);

  if(tally < dosp->limit)
    return;

  in.s_addr = htonl(n->addr.addr.s);
  strcpy(str1, inet_ntoa(in));
  in.s_addr = htonl(n->addr.addr.d);
  sprintf(str, " DoSA port Trig %d- %d (%s->%s: > %d)",
    dosp->no, tally, str1, inet_ntoa(in), dosp->limit);
  log_m('I', NULL, str);
  compose_watch_email_body(email_body, dosp->str, str1);
  email_operator(str, email_body);
}

/**************************************************************************
* scan watch_bt
**************************************************************************/
PRIVATE void scan_tree(treap_t *t)
{

  if(t == NULL)
    return;

  scan_tree(t->l);
  count_ports(TREAP_ENTRY(t));
  scan_tree(t->r);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void do_dosa_port(void)
{
  ll_t *ll;

  log_m('I', NULL, " doing DoSA detection (port)");
  ll = boot.dosa_port_ll;
  for(;;)
  {
    dosp = ll_read_entry(&ll);
    if(dosp == NULL)
      break;

    if(!test_zone(dosp->zone))
      continue;

    scan_tree(watch_bt);
  }
}
